#' Funci\enc{ó}{o}n que proyecta y regionaliza las tasas de mortalidad para el per\enc{í}{i}odo de proyecci\enc{ó}{o}n
#'
#' Esta función proyecta y regionaliza las tasas de mortalidad para el espacio de proyección, sexo, edad y todo el período de proyección. 
#' Para ello, utiliza datos de mortalidad de n períodos anteriores al primer período de la proyección.
#'
#' @param m.data data frame que contiene el espacio de proyección, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y las defunciones.
#' @param l.sup supervivientes proyectados para el espacio superior. Este parámetro se puede introducir o bien calcular con la función \code{\link[demogal]{PMortality}}.
#' @param relation data frame opcional que contiene las relaciones jerárquicas entre el espacio de la proyección y el espacio superior (primera columna, el espacio de la proyección; segunda columna, el espacio superior al de proyección). Las etiquetas de las filas serán las etiquetas del espacio de proyección.
#' @param reg valor alfanumérico de longitud 1 que indica el tipo de espacio bajo el que se aplican los valores de Coale-Demeny. Los valores posibles son W=OESTE, E=ESTE,
#' N=NORTE y S=SUR. Si no se especifica, se toma el valor W que es el que corresponde a España.
#' @keywords demography
#' @export
#' @importFrom stats lm
#' @importFrom stats coef
#' @seealso \code{\link[demogal]{PFertility}}, \code{\link[demogal]{PMortality}}.
#' @return m.p array con las tasas de mortalidad proyectadas (espacio*sexo*edad*tiempo)
#' @references Instituto Galego de Estatística 
#' @encoding UTF-8
#' @examples
#' 
#' nn=15
#' l.sup=PMortality(m.data.s,nn,reg="W")[[2]]
#' m=PMortalityReg(m.data,l.sup,relation,reg="W")


PMortalityReg<- function(m.data, l.sup, relation, reg="W") {
  #Ordenamos el dataframe por si no está ordenado
  m.data<-m.data[order(m.data[,1],m.data[,2],m.data[,3],m.data[,4]),]
  #Generamos un vector con la longitud de los intervalos
  Interval <-c(diff(unique(m.data[, 3])),diff(unique(m.data[,3]))[length(unique(m.data[,3]))-1])
  #Ordenamos los valores proyectados para los supervivientes
  l.sup <- l.sup[order(l.sup[, 4], l.sup[, 3], l.sup[, 2], l.sup[, 1]), ]
  ### Generamos el dataframe de datos del espacio superior
  data_aux = merge(relation, m.data, by.x = names(relation)[1], by.y = names(m.data)[1], all.x = TRUE)
  m.data.sup = aggregate(data_aux[, c(6, 7)],
                         by = list(data_aux[,2],data_aux[, 3], data_aux[, 4], data_aux[, 5]), sum)
  names(m.data.sup)[1:4] = names(data_aux)[c(2, 3, 4, 5)]
  ##### GENERAMOS LAS TABLAS CON LOS VALORES DE TMI Y A POR DEFECTO:
  mort=mortable(m.data, reg=reg)
  mort.sup=mortable(m.data.sup, reg=reg)
  ##### NOS QUEDAMOS CON LAS l
  lt.array=aperm(dropx(mort[ , , , ,9, drop=FALSE],5), c(1,4,2,3))
  lt.array.sup=aperm(dropx(mort.sup[ , , , ,9,drop=FALSE],5), c(1,4,2,3))
  #Calculamos las medias de los 4 años (o los que tengamos)
  l.mean <- apply(lt.array, c(1, 3, 4), FUN = "mean")
  l.mean.sup <- apply(lt.array.sup, c(1, 3, 4), FUN = "mean")
  #Calculamos las transformadas logísticas
  l.mean1 = dropx(xar(l.mean[ , ,1,drop=FALSE],3,dim(l.mean)[3]), 4 )
  log.l = ifelse(l.mean1 - l.mean == 0,log(1e-05/l.mean)/2, log((l.mean1 - l.mean)/l.mean)/2 )
  l.mean.sup1 = dropx(xar(l.mean.sup[ , ,1,drop=FALSE],3,dim(l.mean.sup)[3]),4 )
  log.l.sup = log((l.mean.sup1 - l.mean.sup)/l.mean.sup)/2
  #Calculamos los coeficientes de la regresión entre las transformadas logísticas para las edades comprendidas entre 40 y 99
  limites = array(NA, dim = c(dim(mort)[1:2],1), dimnames=c(dimnames(mort)[1:2],"limit"))
  r2 = array(NA, dim = c(dim(mort)[1:2],1), dimnames=c(dimnames(mort)[1:2],"R2"))            
  coefic = array(NA, dim = c(dim(mort)[1:2],2), dimnames=c(dimnames(mort)[1:2],list(c("alfa", "beta"))))
  rownames(relation)=relation[,1]
  for (i in 1:dim(coefic)[[1]]) {
    for (j in 1:dim(coefic)[[2]]) {
      k = 0
      r = 0
      while (r < 0.95) {
        limit_inf <- findInterval(40, unique(m.data[,3]))
        limit_sup <- findInterval(99, unique(m.data[,3]))
        a = data.frame(
          reg = (log.l[i, j, (limit_inf + k):limit_sup]),
          exp = log.l.sup[dimnames(log.l.sup)[[1]] == relation[dimnames(log.l)[[1]][i], 2], j, (limit_inf + k):limit_sup]
        )
        regre = lm(reg ~ exp, a)
        r = summary(regre)$r.squared
        r2[i, j, ] <- summary(regre)$r.squared
        limites[i, j, ] = k
        coefic[i, j, ] = coef(regre)
        k = k + 1
      }
    }
  }
  
  #Convertimos el data.frame en un array  
  l.sup.array <- array(l.sup[,5], dim = c(length(unique(l.sup[,1])), length(unique(l.sup[, 2])),
                                            length(unique(l.sup[,3])),length(unique(l.sup[, 4]))),
                       dimnames = list(unique(l.sup[,1]), unique(l.sup[, 2]),
                                       unique(l.sup[, 3]), unique(l.sup[,4])))
  l.sup.array1= xar(dropx(l.sup.array[ , ,1, ,drop=FALSE],3),3,dim(l.sup.array)[3])
  log.l.sup = log((l.sup.array1 - l.sup.array)/l.sup.array)/2
  
  coefic1=xar(xar(dropx(coefic[,,1,drop=FALSE],3) ,3,length(unique(m.data[, 3]))),4,length(unique(l.sup[, 4])))
  coefic2=xar(xar(dropx(coefic[,,2,drop=FALSE],3) ,3,length(unique(m.data[, 3]))),4,length(unique(l.sup[, 4])))
  log.l.est= coefic1+coefic2 * log.l.sup[as.character(relation[dimnames(mort)[[1]], ][[2]]) , , ,,drop=FALSE]
  
  #Deshacemos los logit para las proyecciones de los supervivientes:
  l.p = 1e+05/(1 + exp(2 * log.l.est))
  l.p[, , 1, ] = 1e+05
  
  #Calculamos las proyecciones de las defunciones a partir de los supervivientes:
  d.p <- array(NA, dim = c(length(unique(m.data[, 1])), length(unique(m.data[,2])),
                           length(unique(m.data[, 3])), length(unique(l.sup[,4]))),
               dimnames = list(unique(m.data[, 1]), unique(m.data[,2]),
                               unique(m.data[, 3]), unique(l.sup[, 4])))
  d.p[ , ,1:(length(unique(m.data[, 3])) - 1), ]= l.p[ , ,1:(length(unique(m.data[, 3])) - 1), ] -
    l.p[ , ,2:length(unique(m.data[, 3])), ]
  d.p[ , ,length(unique(m.data[, 3])), ] = l.p[, , length(unique(m.data[,3])), ]
  
  #Calculamos las proyecciones de las probabilidades de muerte a partir de las proyecciones de las defunciones y los supervivientes:
  q.p <- d.p/l.p
  
  #Calculamos las proyecciones de las tasas de mortalidad a partir de las probabilidades de muerte proyectadas:
  Intervalx= xar(xar(xar(Interval,1,dim(q.p)[1]),2, dim(q.p)[2]),4,dim(q.p)[4])
  m.p = q.p/(Intervalx * (1 - q.p/2))
  
  return(m.p)
}

