#' Funci\enc{ó}{o}n que proyecta y regionaliza las tasas de fecundidad para el per\enc{í}{i}odo de proyecci\enc{ó}{o}n
#'
#' Esta función se usa para proyectar y regionalizar las tasas de fecundidad para el espacio de proyección, edad y todo el horizonte de proyección. 
#' Utiliza datos de natalidad de n períodos anteriores al primero de la proyección.
#'
#' @param b.data data frame que contiene el espacio de proyección, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y los nacimientos para hacer la proyección de las tasas. Se necesitan n períodos de tiempo (espacio*edad*tiempo*poblacion*evento).
#' @param tfr.sup data frame que contiene el espacio, el período de tiempo y el Índice sintético de fecundidad proyectado y base del espacio superior para el período necesario para proyectar (espacio*tiempo*indice). Este parámetro se puede facilitar o calcular con la función \code{\link[demogal]{PFertility}}
#' @param nn número de períodos de tiempo que se van a proyectar.
#' @param relation data frame opcional que contiene las relaciones jerárquicas entre el espacio de la proyección y el espacio superior (primera columna, el espacio de la proyección; segunda columna, el espacio superior al de proyección). Las etiquetas de las filas serán las etiquetas del espacio de proyección.
#' @keywords demography
#' @export
#' @importFrom reshape melt
#' @importFrom stats runmed
#' @seealso \code{\link[demogal]{PFertility}}, \code{\link[demogal]{PMortality}}.
#' @return F_p array con las tasas de fecundidad proyectadas (espacio*edad de la madre*tiempo)
#' @references
#' Zeng Yi, Wang Zhenglian, Ma Zhongdong e Chen Chunjun. (2000). “A simple method for projecting or estimating alpha and beta: An extension of the Brass Relational Gompertz Fertility Model”, en Population Research and Policy Review 19: 525-549
#' @encoding UTF-8 
#' @examples
#' 
#' nn=15
#' m=PFertilityReg(b.data,tfr.sup=NULL,nn,relation)
#' 



PFertilityReg<-function(b.data,tfr.sup=NULL,nn,relation) {
  year=max(b.data[,3])+1
  rownames(relation)=relation[,1]
  n=length(unique(b.data[,3]))
  ages=as.numeric(as.character(unique(b.data[,2])))
  dif=diff(ages)
  
  #Calculamos el índice sintético de fecundidad
  
  if (is.null(tfr.sup)){ 
    aux2=merge(b.data,relation,by.x=names(b.data)[1],by.y=names(relation)[1],all.x=TRUE)
    b.data.prov=aggregate(aux2[,c(4,5)], list(aux2[,6],aux2[,2],aux2[,3]),sum)
    names(b.data.prov)=names(b.data)
    
    tfr.sup=PFertility(b.data.prov,nn)[[1]]
  }
  
  isfee=tr(b.data)[[1]]
  isfe=melt(isfee)
  
  #Calculamos o DF
  
  isf_pr=subset(tfr.sup,as.numeric(tfr.sup[,2])>=year)
  
  isfe=merge(isfe,relation,by.x=names(isfe)[1],by.y=names(relation)[1],all.x=TRUE)             
  isff=merge(isfe,tfr.sup,by.x=names(isfe)[c(2,ncol(isfe))],by.y=names(tfr.sup)[c(2,1)],all.x=TRUE)
  
  isff[,ncol(isff)+1]=isff[,ncol(isff)-1]/isff[,ncol(isff)]
  
  df=isff[,c(3,1,6)]
  
  df<-df[order(as.numeric(df[,2]),as.numeric(as.character(df[,1]))),1:ncol(df)]
  
  #Calculo dos df proxectados
  pr=project(nn,df)
  
  #Axustamos cos datos do INE os isf proxectados
  isf_pr=isf_pr[order(isf_pr[,2],isf_pr[,1]),]
  
  isf_prr=matrix(isf_pr[,3],length(unique(isf_pr[,1])),length(unique(isf_pr[,2])),dimnames=list(unique(isf_pr[,1]),unique(isf_pr[,2])))
  for ( i in 1:nrow(pr)){
    pr[i,]=pr[i,]*isf_prr[rownames(isf_prr)==relation[rownames(pr)[i],2],]
    
  }
  
  #Calculamos as taxas específicas
  taxas_esp=tr(b.data)[[3]]
  
  #Aplicamos a a función F  a las tasas específicas
  FF=F(taxas_esp,dif)
  
  #Suavizamos as taxas específicas
  #s=suav_taxas2(taxas_esp,pop.fem,ages)
  s=suav_taxas(taxas_esp,n)
  
  # Calculamos a F suavizada das taxas
  F_tilde=F(s,dif)
  
  ####
  
  #Calculamos o Rango intercuartílico e o proxectado
  
  ri=tr(b.data)[[2]]
  rii=melt(ri)
  ri_pr=project(nn,rii)
  
  #Facemos unha matriz co rango intercuartílico do último ano que conocemos e cos proxectados
  ri=cbind(ri[,ncol(ri)],ri_pr)
  
  #Calculo os betas
  beta=ri[,1:(ncol(ri)-1)]/ri[,2:ncol(ri)]
  dim(beta)=c(dim(ri)[1],dim(ri)[2]-1)
  
  # Calculamos e proxectamos a Idade mediana á maternidade
  
  data=b.data[,c(1:3,5)] 
  Emed=medianasg(data)
  Emdd=melt(Emed)
  Emed_pr=project(nn,Emdd)
  Emd=cbind(Emed[,ncol(Emed)],Emed_pr)
  
  #Calculamos os intervalos onde se atopan a mediana e o límite inferior deses intervalos
  inter_me=int(Emd,ages)[[1]]
  li=int(Emd,ages)[[2]]
  
  
  #Collemos a función F do último ano que empregamos para proxectar
  t_ult=FF[,,n]
  dim(t_ult)=c(dim(FF)[1],dim(FF)[2])
  
  
  #Matriz que ten o último isf coñecido e o proxectado
  isf_p=cbind(isfee[,ncol(isfee)],pr)
  
  
  #Proxectamos a F
  
  F_p.acum=F_prox(nn,t_ult,isf_p,inter_me,li,beta,Emd,ages)
  
  #Deshacemos los acumulados y dividimos por la longitud de los intervalos:
  
  F.p<-array(NA,dim=c(length(unique(b.data[,1])),length(unique(b.data[,2])),nn),dimnames<-list(unique(b.data[,1]),unique(b.data[,2]),((year):(year+nn-1))))
  
  for(i in (1:length(unique(b.data[,1])))){
    for(j in (2:length(unique(b.data[,2])))){
      for (k in (1:nn)){
        F.p[i,1,k]=F_p.acum[i,1,k]/diff(ages)[1]
        F.p[i,j,k]=(F_p.acum[i,j,k]-F_p.acum[i,j-1,k])/diff(ages)[j-1]
      }}}
  
  #Si obtemos algunha taxa menor que 0 a igualamos a 0
  F.p[F.p<0]=0
  
  return(F.p)}




##########FUNCIONES EMPLEADAS

#Función que calcula a Función F de Gomperzt

F=function(taxas_esp,dif){
  u=dim(taxas_esp)
  a=array(0,c(u[2],u[1],u[3]))
  for (i in 1:u[3]) {
    a[,,i]=t((taxas_esp)[,,i])
  }
  Ff=apply(a,c(2,3),cumsum)
  b=array(0,c(u[1],u[2],u[3]))
  
  for (i in 1:u[3]) {
    
    b[,,i]=t((Ff)[,,i])*c(dif,dif[length(dif)])
  }
  dimnames(b)=dimnames(taxas_esp)
  return(b)
}


#Función que se emprega para suavizar as taxas
suav_taxas=function(taxas_esp,n){
  s=array(0,dim(taxas_esp))
  su=matrix(0,dim(taxas_esp)[1],dim(taxas_esp)[2])
  t=1
  i=1
  for (t in 1:n){
    for (i in 1:dim(taxas_esp)[1]){
      taxas_s=taxas_esp[,,t] 
      dim(taxas_s)=c(dim(taxas_esp)[1],dim(taxas_esp)[2])
      a=runmed(taxas_s[i,],k= 7)
      su[i,]=a}
    s[,,t]=su 
  }
  return(s)
}
#Función que calcula o intervalo onde está la mediana e o límite inferior dos intervalos

int=function(Emd,ages){
  # Predeclaración CRAN check
  # nn <- NULL
  # ·························
  
  a=matrix(0,nrow(Emd),nn + 1)
  i=1
  for (i in 1:ncol(Emd)){
    
    a[,i]=t(findInterval(Emd[,i],ages)) 
    
  }
  li=matrix(0,nrow(Emd),nn+1)
  for ( i in 1:nrow(a)){
    for (j in 1:ncol(a)){
      for (k in 1:length(ages)){
      li[i,j]=ifelse(a[i,j]==k,ages[k],li[i,j])
      }
      }
  }
  rownames(a)=rownames(Emd)
  rownames(li)=rownames(Emd)
  salida=list(a,li)
  return(salida)
}


#Función para calular as F proxectadas

F_prox=function(nn, t_ult,isf_p,a,li,beta,Emd,ages){
  
  F_proxectada=array(0,c(nrow(t_ult),ncol(t_ult),nn))
  dif=diff(ages)
  dif=mean(dif)
  t=1
  for (t in 1:(nn)){ 
    FEMd=matrix(0,nrow(t_ult),1)
    for ( i in 1:nrow(t_ult)){
      if (a[i,t]<2){FEMd[i]=t_ult[i,(a[i,t])]*(Emd[i,t]-li[i,t])/dif}
      if (a[i,t]>=2) {FEMd[i]=t_ult[i,(a[i,t]-1)]+(t_ult[i,(a[i,t])]-t_ult[i,(a[i,t]-1)])*(Emd[i,t]-li[i,t])/dif}
    }
    
    #Cálculo de alfat
        
    alfa=-log(-log(0.5))-beta[,t]*(-log(-log(FEMd/isf_p[,t])))
    t_ult[t_ult==0]=0.000000001
    aa=-exp(-alfa[,1])*(log(isf_p[,t]/t_ult))^(beta[,t])
    aa[aa=='NaN']=0
    F_proxectada[,,t]=isf_p[,(t+1)]*exp(aa)
    t_ult=F_proxectada[,,t]
    dim(t_ult)=dim(F_proxectada)[1:2]
    
  }
  
  return(F_proxectada)
}

#' Funci\enc{ó}{o}n que proyecta el \enc{Í}{I}ndice sint\enc{é}{e}tico de fecundidad y las tasas de fecundidad para el espacio geogr\enc{á}{a}fico.
#'
#' Esta función proyecta el Indice sintético de fecundidad y las tasas de fecundidad para el espacio geográfico. Para la proyección supone una relación log-funcional entre las mismas y el tiempo. 
#'
#' @param b.data data frame que contiene el espacio geográfico, los extremos inferiores de los intervalos de edad de la madre, el período de tiempo, la población femenina y los nacimientos para los últimos n períodos (espacio*edad madre*tiempo*poblacion*evento).
#' @param nn número de períodos que se van a proyectar.
#' @keywords demography 
#' @export
#' @importFrom plyr adply
#' @importFrom reshape melt
#' @seealso \code{\link[demogal]{PFertilityReg}}, \code{\link[demogal]{PMortalityReg}}.
#' @return tfr.sup data frame que contiene el espacio de proyección, el período de tiempo de la proyección y el índice sintético de fecundidad. 
#' @return tf array con las tasas de fecundidad proyectadas (espacio*edad madre*tiempo)
#' @references Instituto Galego de Estatística
#' @encoding UTF-8
#' @examples
#' 
#' nn=15
#' aux2=merge(b.data,relation,by.x=names(b.data)[1],by.y=names(relation)[1],all.x=TRUE)
#' b.data.prov=aggregate(aux2[,c(4,5)], list(aux2[,6],aux2[,2],aux2[,3]),sum)
#' names(b.data.prov)=names(b.data)
#' m=PFertility(b.data.prov,nn)
#' 
#' 

PFertility=function(b.data,nn){
  year=max(b.data[,3])+1
  n=length(unique(b.data[,3]))
  ages=as.numeric(as.character(unique(b.data[,2])))
  if (0 %in% ages) {dif=c(diff(ages),mean(diff(ages)))}
  if (!(0 %in% ages)) {dif=c(mean(diff(ages)), diff(ages))}    
  
  t=tr(b.data)[[3]]
  tt=adply(t,c(1,2,3))
  ue=as.numeric(as.character(unique(tt[,1])))
  
  tf=array(0,c(length(ue),length(ages),nn),dimnames=list(ue,ages,year:(year+nn-1)))
  aa=matrix(0,length(ue),nn)
  j=1
  i=ue[1]
  for (i in ue){
    uu=subset(tt,as.numeric(as.character(tt[,1]))==i)
    tf[j,,]=project(nn,uu[,-1])
    aa[j,]=colSums(tf[j,,]*dif)
    j=j+1
  }
  colnames(aa)[1:nn]=(year):(year+nn-1)
  rownames(aa)=ue
  
  isfo=tr(b.data)[[1]]
  tfr.sup=rbind(melt(isfo),melt(aa))
  result=list(Total.fertility.rate=tfr.sup,Age.specific.fertility.rate=tf)
}


