#' DEMOGAL
#'
#' La elaboración de indicadores demográficos y proyecciones de población es una tarea habitual 
#' de las oficinas de estadística pública porque, en todo tipo de planificación, es fundamental 
#' conocer la estructura demográfica actual y las perspectivas de evolución de la población.
#' Hoy en día existen herramientas informáticas que permiten el cálculo de indicadores y de 
#' las tablas de mortalidad de manera sencilla, como pueden ser EPIDAT, hojas de cálculo, etc. 
#' Estas herramientas, en algunos casos, no tienen la flexibilidad necesaria para adaptarse
#' a cualquier metodología de cálculo, en otros casos, no permiten procesar grandes conjuntos
#' de datos y, lo que es más importante, cuando se maneja gran cantidad de información, la mayor
#' parte de ellas no permiten automatizar el proceso de cálculo, etc. 
#' Teniendo en cuenta todas estas dificultades que plantean las herramientas existentes,
#' desde el Instituto Galego de Estatística (IGE) se pensó en aprovechar la flexibilidad de R 
#' para desarrollar un paquete que permitiese automatizar el cálculo de las funciones
#' demográficas que periódicamente se utilizan en el IGE para calcular indicadores y para 
#' proyectar flujos y stocks poblacionales. 
#'  
#' El método de este paquete para proyectar los flujos y stocks poblacionales parte de la población residente en un cierto ámbito geográfico (provincias, comarcas, etc.) y de los datos
#' observados para cada uno de las componentes demográficas básicas, la mortalidad, la fecundidad y la migración. 
#' Se trata de obtener la población correspondiente a fechas o períodos posteriores bajo ciertas hipótesis sobre la evolución de estos
#' tres fenómenos. 
#' La elaboración de estas proyecciones de población está basada en el método clásico de componentes, el cual, a partir de una población residente en un cierto espacio geográfico y de los 
#' datos observados para cada una de las componentes demográficas básicas (mortalidad, fecundidad y migración), permite obtener la población en períodos futuros bajo ciertas hipótesis 
#' sobre la evolución de las componentes, que son las que determinan el crecimiento de la población y su estructura por edades.
#' Se utiliza el modelo de proyección multirregional propuesto por Willekens, que proporciona las cifras de población residente y las cifras proyectadas de nacimientos, defunciones y 
#' movimientos migratorios que tendrán lugar en cada espacio considerado y para cada uno de los períodos del horizonte proyectivo, guardando todo ello la necesaria coherencia entre 
#' flujos y stocks demográficos y la debida consistencia interterritorial.
#' En este paquete se contemplan dos enfoques:
#' \itemize{
#'\item Proyección de la población para uno o más espacios geográficos aplicando el modelo multirregional.
#'\item Proyección de la población para varios espacios geográficos mediante el modelo multirregional, pero imponiendo que los datos agregados del conjunto de espacios coincidan con los de una proyección preexistente del área formada por el conjunto de los espacios (consistencia con el espacio superior).
#' }
#'
#' Por tanto, se puede disponer de un espacio o varios que se desean proyectar y también se puede disponer de un espacio superior o varios. El método empleado permite
#' hacer una proyección coherente con los datos de los espacios superiores. En este último caso, el espacio o espacios superiores deben estar relacionados
#' jerárquicamene con los espacios de la proyección, por ejemplo, las provincias de Galicia como espacios de proyección y Galicia como espacio superior.
#' Como resumen, el método programado permite ajustar (si se desea) los datos finales a las posibles proyecciones disponibles de un espacio superior.
#'
#' Este método empleado proyecta cada uno de los fenómenos demográficos por separado, para después unirlos en la obtención de la proyección de la población. 
#' Las hipótesis que se hayan planteado sobre cada uno de estos fenómenos demográficos se aplican a la población en el período de partida $t_0$, obteniendo la población en el
#' período $t_{0+1}$. Actuando así, de forma iterada, se llega al horizonte de la proyección, $t_{0+nn}$.
#' Para la obtención de la proyección de la población será necesario disponer de información de nacimientos, defunciones y movimientos migratorios correspondiente a los últimos $n$ períodos: $t_{0-n}, t_{0-(n-1)}, \ldots , t_{0-1}$.
#' 
#' El desarrollo de demogal está inspirado en la metodología de proyección empleada por el INE (2010).
#' 
#' En el ejemplo que se incluye en este paquete el objetivo es la proyección de la población de las 53 comarcas gallegas, tomando como base la población de 1 de enero del 2013 y proyectando la población para un horizonte de 15 años. Se utiliza como espacio superior las 4 provincias gallegas.
#' @references
#' Durbán, M. (2012). Splines con Penalizaciones: Teoría y aplicaciones. Universidad Carlos III de Madrid. 
#' @references 
#' Willekens, F.J. and Drew, P. (1984), A Multiregional Model for Regional Demographic Projection. Demographic
#' Research and Spatial Policy, Ch.15, pp. 309-331 - Academic Press, London.
#' @references
#' Epidat: programa para análise epidemiolóxica de datos. Versión 4.0, xuño 2012. Consellería de Sanidade, Xunta de Galicia, España; Organización Panamericana de la Salud (OPS-OMS). Disponible en: http://dxsp.sergas.es
#' @references
#'  Vinuesa, J. y Puga, D. (2007), Técnicas y ejercicios de Demografía
#' @references
#'  Zeng Yi, Wang Zhenglian, Ma Zhongdong e Chen Chunjun. (2000). “A simple method for projecting or estimating alfa and beta: An extension of the Brass Relational Gompertz Fertility Model”, en Population Research and Policy Review 19: 525-549  
#' @references
#' Rob J Hyndman with contributions from Heather Booth, Leonie  Tickle and John Maindonald. (2014). demography: Forecasting   mortality, fertility, migration and population data. R package version 1.18. https://CRAN.R-project.org/package=demography
#' @references
#' INE (2010). Proyección de la Población de España a Corto Plazo. Metodología
#' @name demogal
#' @aliases demogal
#' @encoding UTF-8
#' @docType package
NULL


# Data set information
#' Dataset de nacimientos
#'
#' Data frame con los datos de nacimientos por comarcas en Galicia empleado para la proyección futura de las tasas de fecundidad.
#'
#' \itemize{
#' \item espacio: etiqueta del espacio de la proyección.
#' \item edad: extremos inferiores de los intervalos de edad de la madre.
#' \item tiempo: año en el que ocurrieron los nacimientos.
#' \item pop: población femenina en edad fértil.
#' \item datos: número de nacimientos.
#' }
#'
#' @docType data
#' @keywords datasets
#' @name b.data
#' @usage b.data
#' @encoding UTF-8
#' @format Un data frame con 3710 en filas y 5 variables en columnas
NULL

#' Dataset de mortalidad
#'
#' Data frame con los datos necesarios para calcular la tabla de mortalidad por comarcas en Galicia.
#'
#' \itemize{
#' \item espacio etiqueta del espacio de la proyección.
#' \item sexo etiqueta del sexo
#' \item edad extremos inferiores de los intervalos de edad en los que se produce las defunciones.
#' \item tiempo año en el que ocurrieron las defunciones.
#' \item P población 
#' \item D defunciones
#' }
#'
#' @docType data
#' @keywords datasets
#' @name m.data
#' @usage m.data
#' @encoding UTF-8
#' @format Un data frame con 9328 en filas y 6 variables en columnas
NULL

#' Dataset de mortalidad
#'
#' Data frame con los datos necesarios para calcular la tabla de mortalidad del espacio superior (provincias de Galicia) con 10 años.
#'
#' \itemize{
#' \item espacio etiqueta del espacio de la proyección.
#' \item sexo etiqueta del sexo
#' \item edad extremos inferiores de los intervalos de edad de la defunción.
#' \item tiempo años en los que ocurrieron las defunciones.
#' \item P población 
#' \item D defunciones
#' }
#'
#' @docType data
#' @keywords datasets
#' @name m.data.s
#' @usage m.data.s
#' @encoding UTF-8
#' @format Un data frame con 1936 en filas y 6 variables en columnas
NULL

#' Dataset de la poblacion base de la proyeccion
#'
#' Data frame con los datos de la población base a 1 de enero por comarcas de Galicia y de la que se parte para proyectar.
#'
#' \itemize{
#' \item espacio etiqueta del espacio de la proyección.
#' \item sexo etiqueta del sexo
#' \item edad extremos inferiores de los intervalos de edad de la población.
#' \item tiempo año de la población.
#' \item pob población 
#' }
#'
#' @docType data
#' @keywords datasets
#' @name pop
#' @usage pop
#' @encoding UTF-8
#' @format Un data frame con 2332 en filas y 5 variables en columnas
NULL


#' Dataset de los emigrantes al exterior
#'
#' Data frame con los datos de los emigrantes de las comarcas de Galicia al exterior para los últimos 3 años.
#'
#' \itemize{
#' \item espacio etiqueta del espacio de la proyección.
#' \item sexo etiqueta del sexo
#' \item edad extremos inferiores de los intervalos de edad de la población.
#' \item tiempo año de los emigrantes al exterior.
#' \item pop población.
#' \item em emigrantes exteriores.
#' }
#'
#' @docType data
#' @keywords datasets
#' @name emig.out
#' @usage emig.out
#' @encoding UTF-8
#' @format Un data frame con 6996 en filas y 6 variables en columnas
NULL

#' Dataset de los inmigrantes procedentes del exterior
#'
#' Data frame con los datos de los inmigrantes de las comarcas de Galicia procedentes del exterior para los últimos 3 años.
#'
#' \itemize{
#' \item espacio etiqueta del espacio de la proyección.
#' \item sexo etiqueta del sexo
#' \item edad extremos inferiores de los intervalos de edad de la población.
#' \item tiempo año de las inmigraciones del exterior.
#' \item pop población.
#' \item im inmigrantes. 
#' }
#'
#' @docType data
#' @keywords datasets
#' @name imig.out
#' @usage imig.out
#' @encoding UTF-8
#' @format Un data frame con 8863 en filas y 6 variables en columnas
NULL

#' Dataset de los emigrantes interiores
#'
#' Data frame con los datos de los emigrantes interiores entre las comarcas de Galicia en los últimos 3 años.
#'
#' \itemize{
#' \item espacio etiqueta del espacio de la proyección.
#' \item sexo etiqueta del sexo
#' \item edad extremos inferiores de los intervalos de edad de la población.
#' \item tiempo año de las emigraciones interiores.
#' \item pop población.
#' \item int emigrantes interiores 
#' }
#'
#' @docType data
#' @keywords datasets
#' @name emig.int
#' @usage emig.int
#' @encoding UTF-8
#' @format Un data frame con 6996 en filas y 6 variables en columnas
NULL

#' Dataset de los emigrantes interiores segun origen y por destino
#'
#' Data frame con los datos de la suma de los emigrantes interiores entre las comarcas de Galicia en los últimos 3 años.
#'
#' \itemize{
#' \item espacio etiqueta del espacio origen de la emigración interna.
#' \item sexo etiqueta del sexo
#' \item edad extremos inferiores de los intervalos de edad de la población.
#' \item espaciodestino etiqueta del espacio destino de la emigración interna.
#' \item int emigrantes interiores 
#' }
#'
#' @docType data
#' @keywords datasets
#' @name emig.intc
#' @usage emig.intc
#' @encoding UTF-8
#' @format Un data frame con 25274 en filas y 5 variables en columnas. Las etiquetas de las filas deben ser las etiquetas del espacio de proyección.
NULL

#' Dataset de relacion entre el espacio de la proyeccion y el espacio superior
#'
#' Data frame con los datos de relación entre el espacio superior (provincias de Galicia) y cada espacio de la proyección (comarcas de Galicia).
#'
#' \itemize{
#' \item espacio etiqueta del espacio de la proyección.
#' \item espaciosup etiqueta del espacio superior de la proyección.
#' }
#'
#' @docType data
#' @keywords datasets
#' @name relation
#' @usage relation
#' @encoding UTF-8
#' @format Un data frame con 53 en filas y 2 variables en columnas
NULL

#' Dataset con los datos de la proporcion de poblacion que pasa de un grupo de edad a otro 
#'
#' Dataset con los datos de la proporción de población, promedio de los últimos n años anteriores a la proyección, que pasa de un grupo de edad a otro en al año siguiente (espacio*sexo*edad) según las comarcas de Galicia, sexo y grupo de edad. 
#'
#' \itemize{
#' \item espacio etiqueta del espacio de la proyección.
#' \item sexo etiqueta del sexo
#' \item edad extremos inferiores de los intervalos de edad de la población.
#' \item factor proporción de población promedio de los últimos 10 años anteriores a la proyección que pasa de un grupo de edad a otro al año siguiente.
#' }
#'
#' @docType data
#' @keywords datasets
#' @name r
#' @usage r
#' @encoding UTF-8
#' @format Un data frame con 2120 en filas y 4 variables en columnas
NULL
