#' Funci\enc{ó}{o}n principal que proyecta la poblaci\enc{ó}{o}n, nacimientos y defunciones
#'
#' Esta función proyecta la población, los nacimientos y las defunciones a partir de datos de población, nacimientos, defunciones y migración de períodos anteriores. 
#'
#' @param nn número de períodos que se van a proyectar.
#' @param pop data frame que contiene el espacio de proyección, el sexo, los extremos inferiores de los intervalos de edades, el tiempo y la población base de la proyección (espacio*sexo*edad*tiempo*poblacion).
#' @param b.data data frame que contiene el espacio de proyección, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y los nacimientos para proyectar las tasas de fertilidad. Se necesitan n períodos de tiempo (espacio*edad*tiempo*poblacion*evento).
#' @param m.data data frame que contiene el espacio, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y las defunciones. Se necesitan n períodos de tiempo. 
#' @param imig.out data frame que contiene el espacio, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y los inmigrantes procedentes del exterior (espacio*sexo*edad*tiempo*poblacion*evento).Se necesita disponer de n períodos de tiempo.
#' @param emig.out data frame que contiene el espacio, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y los emigrantes al exterior (espacio*sexo*edad*tiempo*población*evento). Se necesita disponer de n períodos de tiempo.
#' @param emig.int data frame que contiene el espacio de proyección, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y los emigrantes interiores (espacio*sexo*edad*tiempo*poblacion*evento). Se necesita disponer de n períodos de tiempo.
#' @param emig.intc data frame con la suma de los emigrantes interiores del espacio origen h al espacio destino k en los últimos n períodos (espacio origen*sexo*edad*espacio destino)
#' @param m.data.s data frame opcional que contiene el espacio superior, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y las defunciones para n períodos.
#' @param v data frame opcional cuya primera columna es el espacio o el espacio superior y la segunda y sucesivas, los datos de la inmigración procedente del exterior para el período. Si la inmigración se considera constante durante todo el período de proyección es suficiente con introducir una columna. 
#' @param r data frame opcional que contiene el espacio de proyección, el sexo, los extremos inferiores de los intervalos de edades y la proporción de población promedio de los últimos n períodos de tiempo anteriores a la proyección que pasa de un grupo de edad a otro en el período siguiente (espacio*sexo*edad*proporción).
#' @param relation data frame opcional que contiene las relaciones jerárquicas entre el espacio de la proyección y el espacio superior (primera columna, el espacio de la proyección; segunda columna, el espacio superior al de proyección). Las etiquetas de las filas serán las etiquetas del espacio de proyección.
#' @param pop.sup.pr data frame opcional que contiene el espacio superior, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo y la población proyectada para el espacio superior (espacio superior*sexo*edad*tiempo*población).
#' @param b.sup.pr data frame opcional que contiene el espacio superior, el sexo, el período de tiempo y los nacimientos proyectados para el espacio superior (espacio superior*sexo*tiempo*nacimientos).
#' @param m.sup.pr data frame opcional que contiene el espacio superior, el sexo, el período de tiempo y las defunciones proyectadas para el espacio superior (espacio superior*sexo*edad*tiempo*defunciones).
#' @param reg valor alfanumérico de longitud 1 que indica el tipo de espacio bajo el que se aplican los valores de Coale-Demeny. Los valores posibles son W=OESTE, E=ESTE,
#' N=NORTE y S=SUR. Si no se especifica, se toma el valor W que es el que corresponde a España.
#' @keywords demography
#' @export
#' @seealso \code{\link[demogal]{PMortality}}, \code{\link[demogal]{PMortalityReg}}, \code{\link[demogal]{PFertility}}, \code{\link[demogal]{PFertilityReg}}.
#' @return population data frame con la población proyectada (espacio*sexo*edad*tiempo*poblacion)
#' @return births data frame con los nacimientos proyectados (espacio*sexo*tiempo*nacimientos)
#' @return deaths data frame con las defunciones proyectadas (espacio*sexo*edad*tiempo*defunciones)
#' @references Instituto Galego de Estatística
#' @encoding UTF-8
#' @examples
#' nn=15
#' b=demogal.proj(nn,pop,b.data,m.data,emig.out,imig.out,emig.int,emig.intc,r=NULL,relation,m.data.s)


demogal.proj=function(nn,pop,b.data,m.data,emig.out,imig.out,emig.int=NULL,emig.intc=NULL,r=NULL,relation=NULL,m.data.s=NULL,pop.sup.pr=NULL,m.sup.pr=NULL,b.sup.pr=NULL,v=NULL,reg="W"){
  ll=length(unique(pop[,3]))
  p=1/(sum(diff(unique(pop[,3])))/ll)
  year=pop[,4][1]
  if (is.null(r)){
    r<-expand.grid(unique(pop[,1]),unique(pop[,2]),unique(pop[,3])[2:(ll-1)])
    r[,4]=p
    names(r)=c("espazo","sexo","Idadeq","dato")}
  
  if (is.null(relation)){
    m.rate.pr=PMortality(m.data, nn)[[1]]
    f.rate.pr=PFertility(b.data,nn)[[2]]
    u=ExtEm(emig.out,nn)
    uin=ExtIn(imig.out,relation=NULL,nn,v_inmig=v)
    
    
    #Lista con la emigraciones y las inmigraciones. En este caso no hay migraciones internas
    if (length(unique(pop[,1])==1)){
      
      migration=list(u[[1]],uin,NULL)}
    else{
      migration=list(u[[1]],uin,NULL)
      if (!is.null(emig.intc)){
        w=IntEm(emig.int,emig.intc,nn)
        migration=list(u[[1]],uin,w)}
    }
    m=projection(nn,r,pop,m.rate.pr,f.rate.pr,migration,relation=NULL)}
  
  if (!is.null(relation)){
    
    ### SE PROYECTA LA POBLACIÓN 
    l.sup=PMortality(m.data.s,nn,reg=reg)[[2]]
    m.rate.pr=PMortalityReg(m.data,l.sup,relation,reg=reg)
    f.rate.pr=PFertilityReg(b.data,tfr.sup=NULL,nn,relation)
    
    #EMIGRACIONES EXTERNAS
    aux2=merge(emig.out,relation,by.x=names(emig.out)[1],by.y=names(relation)[1],all.x=TRUE)
    emig.outt=aggregate(aux2[,c(5,6)],list(aux2[,7],aux2[,2],aux2[,3],aux2[,4]),sum)
    names(emig.outt)=names(emig.out)
    u=ExtEm(emig.outt,nn)
    iset=u[[2]]
    ise=u[[3]]
    u=ExtEmReg(emig.out,iset,ise,relation,nn)
    
    #INMIGRACIONES PROCEDENDES DEL EXTERIOR
    v=ExtIn(imig.out,relation,nn,v_inmig=v)
    
    #EMIGRACIONES INTERNAS
    if (!is.null(emig.intc)){
      w=IntEm(emig.int,emig.intc,nn)
      migration=list(u,v,w)}
    else
    {migration=list(u,v,NULL)}
    m=projection(nn,r,pop,m.rate.pr,f.rate.pr,migration,relation,pop.sup.pr,b.sup.pr,m.sup.pr)
    
  }
  
  pp=m[[1]]
  p=array(0,dim=c(dim(pp[[1]]),length(pp)))
  dimnames(p)[1:3]=dimnames((pp)[[1]])
  dimnames(p)[[4]]=((year+1):(year+nn))
  for (i in 1:length(pp)){
    p[,,,i]=pp[[i]]
  }
  
  population=adply(p,c(1,2,3,4))
  names(population)=c("espacio","sexo","Edad","Tiempo","poblacion")
  births=adply(m[[2]],c(1,2,3))
  names(births)=c("espacio","sexo","Tiempo","nacimientos")
  mortality=adply(m[[3]],c(1,2,3,4))
  names(mortality)=c("espacio","sexo","Edad","Tiempo","defunciones")
  
  u=list("population"=population,"births"=births,"deaths"=mortality)
  
  return(u)
  
}

