#' Funci\enc{ó}{o}n que proyecta y regionaliza las inmigraciones procedentes del exterior
#'
#' Esta función se usa para proyectar y regionalizar (opcional) el número de inmigrantes procedentes del exterior. En ella se parte de 
#' la hipótesis de que se considera un número predeterminado de inmigrantes durante el período de proyección para 
#' el espacio o espacio superior (v_inmig). Si no se introduce el parámetro, toma la media de los que hubo en los últimos n años.
#'
#' @param inmig.out data frame que contiene el espacio geográfico, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y los inmigrantes procedentes del exterior (espacio*sexo*edad*tiempo*poblacion*evento).
#' @param relation data frame opcional que contiene las relaciones jerárquicas entre el espacio de la proyección y el espacio superior (primera columna, el espacio de la proyección; segunda columna, el espacio superior al de proyección). Las etiquetas de las filas serán las etiquetas del espacio de proyección.
#' @param nn número de períodos que se van a proyectar.
#' @param v_inmig data frame cuya primera columna es el espacio o el espacio superior y la segunda y siguientes, los datos de la inmigracion procedente del exterior para cada uno de los años de la proyección. Si se considera que la inmigración es constante durante todo el período de proyección, es suficiente con introducir una columna.
#' @keywords demography
#' @export
#' @importFrom stats aggregate
#' @importFrom stats loess
#' @seealso \code{\link[demogal]{ExtEm}}, \code{\link[demogal]{IntEm}}.
#' @return tt array con los inmigrantes procedentes del exterior para todo el período de proyección (espacio*sexo*edad*tiempo)
#' @references Instituto Galego de Estatística
#' @encoding UTF-8
#' 



ExtIn=function(inmig.out,relation=NULL,nn,v_inmig=NULL){
  
  year=max(inmig.out[,4])+1
  ind=0
  if (is.null(relation)){
    ind=1
  }
  
  if (is.null(relation)){
    relation=data.frame(espazo=unique(inmig.out[,1]),espazosup=unique(inmig.out[,1]))
    rownames(relation)=unique(inmig.out[,1])}
  
  aux2=merge(inmig.out,relation,by.x=names(inmig.out)[1],by.y=names(relation)[1],all.x=TRUE)
  imig_out_sup=aggregate(aux2[,c(5,6)], list(aux2[,7],aux2[,2],aux2[,3],aux2[,4]),sum)
  names(imig_out_sup)=names(inmig.out)
  
  if (is.null(v_inmig)){
    v_inmig=aggregate(imig_out_sup[,6], by=list(imig_out_sup[,1]), sum)
    v_inmig[,2]=v_inmig[,2]/length(unique(imig_out_sup[,4]))
  }
  
  unano=function(inmig.out,relation,v_inmig,ind){
    ar=d2a(list(sort(unique(inmig.out[,1])),sort(unique(inmig.out[,2])),sort(unique(inmig.out[,3])),sort(unique(inmig.out[,4]))),inmig.out)[[2]]
    
    #distribuimos por sexo
    names(v_inmig)[1]=names(relation)[2] 
    ar2=d2a(list(sort(unique(v_inmig[,1]))),v_inmig)
    ars=apply(ar,c(1,2),sum)/length(unique(imig_out_sup[,4]))
    ar2=matrix(ar2,length(unique(v_inmig[,1])),1)
    rownames(ar2)=unique(v_inmig[,1])
    
    if (ind==1) {aa1=calibdim(ars,ar2)} else {aa1=calimix(ars,ar2,relation)}
 
    #distribuimos por idade calibramos e suavizamos
    ars=array(aa1,dim=c(dim(aa1),1),dimnames=c(dimnames(aa1),"1"))
    arid=apply(ar,c(1,2,3),sum)/length(unique(imig_out_sup[,4]))
    
    aa2=calibdim(arid,ars)
    
    #suavizamos
    
    uus=array(0,dim=dim(aa2),dimnames=dimnames(aa2))
    i=1
    k=1
    for (i in 1:length(dimnames(aa2)[[1]])){
      for (k in 1:length(dimnames(aa2)[[2]])){
        uus[i,k,]=loess(aa2[i,k,]~as.numeric(as.character(names(aa2[i,k,]))))$fitted
      }
    }
    uus[uus<0]=0
    #calibramos lo suavizado
    
   tt=calibdim(uus,ars)
  }
  
  tt=unano(inmig.out,relation,v_inmig[,c(1,2)],ind)
  
  if (ncol(v_inmig)<3){
    ttt=array(0,dim=c(dim(tt),nn),dimnames=list(unique(dimnames(tt)[[1]]),unique(dimnames(tt)[[2]]),unique(dimnames(tt)[[3]]),((year):(year+nn-1))) )
    for (i in 1:nn){
      ttt[,,,i]=tt
    }
  }
  else 
  {
    ttt=array(0,dim=c(dim(tt),nn),dimnames=list(unique(dimnames(tt)[[1]]),unique(dimnames(tt)[[2]]),unique(dimnames(tt)[[3]]),((year):(year+nn-1))) )  
    for (i in 1:nn){
      tt=unano(inmig.out,relation,v_inmig[,c(1,i+1)],ind)
      ttt[,,,i]=tt
    } 
  }
  
  return(ttt) 
}  


#' Funci\enc{ó}{o}n que proyecta las tasas de emigraci\enc{ó}{o}n al exterior
#'
#' Esta función se usa para proyectar las tasas de emigración al exterior del espacio de proyección. En ella se considera que las tasas
#' se pueden descomponer en el producto de dos factores: la intensidad de la emigración exterior, en términos del Índice
#' sintético de emigración por sexo, y el calendario por edad y sexo del fenómeno de la emigración.
#'
#' @param emig.out data frame que contiene el espacio geográfico, sexo, extremos inferiores de los intervalos de edades, período de tiempo, población y emigrantes al exterior (espacio*sexo*edad*tiempo*poblacion*evento).
#' @param nn número de períodos que se van a proyectar.
#' @keywords demography
#' @export
#' @importFrom stats loess
#' @seealso \code{\link[demogal]{ExtIn}}, \code{\link[demogal]{IntEm}}.
#' @return e array con las tasas de emigración al exterior para todo el período de proyección (espacio*sexo*edad*tiempo).
#' @return iset array con el índice sintético de emigración para todo el período de proyección.
#' @return ise array con el índice sintético de emigración por sexo para la media de los períodos utilizados.
#' @references Instituto Galego de Estatística.
#' @encoding UTF-8
#' @examples
#' 
#' nn=15
#' aux2=merge(emig.out,relation,by.x=names(emig.out)[1],by.y=names(relation)[1],all.x=TRUE)
#' emig.out.s=aggregate(aux2[,c(5,6)],list(aux2[,7],aux2[,2],aux2[,3],aux2[,4]),sum)
#' names(emig.out.s)=names(emig.out)
#' u=ExtEm(emig.out.s,nn)
#' 



ExtEm=function(emig.out,nn){
  
  year=max(emig.out[,4])+1
  
  #sumamos por sexo
  emig_fora_t=agregat(emig.out,varnames=names(emig.out)[c(5,6)], byst=names(emig.out)[c(1,3,4)])
  #sumamos por años
  emig_fora_t1=agregat(emig.out,varnames=names(emig.out)[c(5,6)], byst=names(emig.out)[c(1,2,3)])
  emig_fora_t1$ano=year
  ages=as.numeric(as.character(unique(emig_fora_t[,2])))
  emig_fora_t1=emig_fora_t1[,c(1,2,3,6,4,5)]
  
  emig_fora_t2=agregat(emig_fora_t1,varnames=names(emig_fora_t1)[c(5,6)],byst=names(emig_fora_t1)[c(1,3,4)])
  
  ise=tr(emig_fora_t)[[1]]
  ised=tr(emig_fora_t2)[[1]]
  ise1=tr(emig_fora_t1[emig_fora_t1[,2]==1,][,-2])[[1]]
  ise2=tr(emig_fora_t1[emig_fora_t1[,2]==6,][,-2])[[1]]
  
  #CALCULAMOS DIFERENCIAL
  
  de1=(ise1/ised)
  de2=(ise2/ised)
  
  isee=cbind(as.numeric(as.character(rownames(ise))),ise[,ncol(ise)])
  colnames(isee)=c("esp","ise")
  
  de1=cbind(1,de1)
  de2=cbind(6,de2)
  colnames(de1)=colnames(de2)=c("sex","de")
  de=rbind(de1,de2)
  de=cbind(de,u=as.numeric(as.character(rownames(de))))
  
  
  tablabase<-expand.grid(unique(emig.out[,1]),unique(emig_fora_t1[,2]),(year:(year+nn-1)))
  tablabase=merge(tablabase,de, by.x=names(tablabase[,c(1,2)]),by.y=colnames(de[,c(3,1)]), all.x=TRUE)
  names(tablabase)=names(emig_fora_t1)[c(-3,-5)]
  tablabase=merge(tablabase,isee,by.x=names(tablabase)[1],by.y=colnames(isee)[1],all.x=TRUE)
  
  ##ist de provincias
  tablabase$iset= tablabase[,4]*tablabase[,5]
  
  iset=d2a(list(unique(tablabase[,1]),unique(tablabase[,2]),sort(unique(tablabase[,3])) ) , tablabase)[[1]]
  
  
  #PROYECCIÓN DEL CALENDARIO
  
  tise1=tr(emig_fora_t1[emig_fora_t1[,2]==1,][,-2])[[3]]
  tise2=tr(emig_fora_t1[emig_fora_t1[,2]==6,][,-2])[[3]]
  
  
  u=findInterval(85,ages)
  
  taxass=tise1[,(u:length(ages)),]
  dim(taxass)=c(dim(tise1)[1],length(u:length(ages)),dim(tise1)[3])
  taxass=apply(taxass,c(1),mean) 
  for (i in (u:length(ages))){
    tise1[,i,]=taxass
  }
  taxass=tise2[,(u:length(ages)),]
  dim(taxass)=c(dim(tise2)[1],length(u:length(ages)),dim(tise2)[3])
  taxass=apply(taxass,c(1),mean) 
  for (i in (u:length(ages))){
    tise2[,i,]=taxass
  }
  
  cal1=tise1
  cal2=tise2
  for (i in 1:length(unique(dimnames(tise1)[[2]]))){
    cal1[,i,]=tise1[,i,]/ise1
    cal2[,i,]=tise2[,i,]/ise2
  }
  
  
  for (i in 1:length(unique(dimnames(cal1)[[1]]))){
    cal1[i,,]=loess(cal1[i,,]~names(cal1[i,,]))$fitted
    cal2[i,,]=loess(cal2[i,,]~names(cal2[i,,]))$fitted
  }
  cal=array(0,c(length(unique(dimnames(cal1)[[1]])),2,length(unique(dimnames(cal1)[[2]]))),dimnames=list(unique(dimnames(cal1)[[1]]),c("1","6"),unique(dimnames(cal1)[[2]])) )
  cal[,1,]=cal1
  cal[,2,]=cal2 
  ise=array(0,c(length(unique(emig_fora_t[,1])),2),list(unique(emig_fora_t[,1]),c("1","6")))
  ise[,1]=ise1
  ise[,2]=ise2
  e=cal
  for (i in 1:length(dimnames(cal)[[3]])){
    e[,,i]=cal[,,i]*ise 
  }
  ee=array(0,c(dim(e),nn))
  for (i in 1:nn){
    ee[,,,i]=e
  }
  dimnames(ee)[1:3]=dimnames(e)
  dimnames(ee)[[4]]=(year):(year+nn-1)
  result=return(list("Emigration rate"=ee,"Total emigration rate"=iset,"Average emigration rate"=ise))
} 




#' Funci\enc{ó}{o}n que proyecta y regionaliza las tasas de emigraci\enc{ó}{o}n al exterior
#'
#' Esta función se usa para proyectar y regionalizar las tasas de emigración al exterior del espacio de proyección. En ella se considera que las tasas
#' se pueden descomponer como producto de dos factores: la intensidad de la emigración exterior, en términos del Índice
#' sintético de emigración por sexo, y el calendario por edad y sexo del fenómeno de la emigración.
#'
#' @param emig.out data frame que contiene el espacio, sexo, extremos inferiores de los intervalos de edades, período de tiempo, población y emigrantes al exterior (espacio*sexo*edad*tiempo*poblacion*evento).
#' @param iset índice sintético de emigración calculado para el espacio superior y para todo el período de proyección (espacio*sexo*edad*tiempo).
#' @param ise índice sintético de emigración medio calculado para el espacio superior y para los últimos n años  (espacio*sexo*edad*tiempo).
#' @param relation data frame opcional que contiene las relaciones jerárquicas entre el espacio de la proyección y el espacio superior (primera columna, el espacio de la proyección; segunda columna, el espacio superior al de proyección). Las etiquetas de las filas serán las etiquetas del espacio de proyección.
#' @param nn número de períodos que se van a proyectar.
#' @keywords demography
#' @export
#' @importFrom stats loess
#' @seealso \code{\link[demogal]{ExtIn}}, \code{\link[demogal]{IntEm}}.
#' @return e array con las tasas de emigración al exterior para todo el período de proyección (espacio*sexo*edad*tiempo)
#' @references Instituto Galego de Estatística
#' @encoding UTF-8
#' @examples
#' 
#' nn=15
#' aux2=merge(emig.out,relation,by.x=names(emig.out)[1],by.y=names(relation)[1],all.x=TRUE)
#' emig.outt=aggregate(aux2[,c(5,6)],list(aux2[,7],aux2[,2],aux2[,3],aux2[,4]),sum)
#' names(emig.outt)=names(emig.out)
#' u=ExtEm(emig.outt,nn)
#' iset=u[[2]]
#' ise=u[[3]]
#' u=ExtEmReg(emig.out,iset,ise,relation,nn)
#' 
#' 



ExtEmReg=function(emig.out,iset,ise,relation,nn){
  ##calcular datos de comarcas
  year=max(emig.out[,4])+1
  ages=sort(as.numeric(as.character(unique(emig.out[,3]))))
  u=findInterval(85,ages)
  
  ise1=matrix(ise[,1],nrow(ise),1)
  rownames(ise1)=rownames(ise)
  ise2=matrix(ise[,2],nrow(ise),1)
  rownames(ise2)=rownames(ise)
  emig_fora_c=agregat(emig.out,varnames=names(emig.out)[c(5,6)], byst=names(emig.out)[c(1,2,3)])
  emig_fora_c$ano=year
  emig_fora_c=emig_fora_c[,c(1,2,3,6,4,5)]
  ise1c=tr(emig_fora_c[emig_fora_c[,2]==1,][,-2])[[1]]
  ise2c=tr(emig_fora_c[emig_fora_c[,2]==6,][,-2])[[1]]
  
  de1c=ise1c
  de2c=ise2c
  rownames(relation)=relation[,1]
  for (i in 1:length(unique(dimnames(ise1c)[[1]]))){
    de1c[i,]=ise1c[i,]/ise1[rownames(ise1)==relation[rownames(ise1c)[i],2],]
    de2c[i,]=ise2c[i,]/ise2[rownames(ise2)==relation[rownames(ise2c)[i],2],]
  }  
  
  de=cbind(de1c,de2c)
  colnames(de)=c("1","6")
  
  isetp=de
  for (i in 1:length(unique(dimnames(de)[[1]]))){
    isetp[i,]=de[i,]*iset[rownames(iset)==relation[rownames(de)[i],2],,1] }
  
  isetpp=array(0,c(length(unique(dimnames(isetp)[[1]])),length(unique(dimnames(isetp)[[2]])),nn),dimnames=list(unique(dimnames(isetp)[[1]]),unique(dimnames(isetp)[[2]]),(year+1):(year+nn)))
  
  for (i in 1:nn){
    isetpp[,,i]=isetp
  }
  #calendario comarcas
  tise1c=tr(emig_fora_c[emig_fora_c[,2]==1,][,-2])[[3]]
  tise2c=tr(emig_fora_c[emig_fora_c[,2]==6,][,-2])[[3]]
  
  taxass=tise1c[,(u:length(ages)),]
  taxass=apply(taxass,c(1),mean) 
  for (i in (u:length(ages))){
    tise1c[,i,]=taxass
  }
  
  taxass=tise2c[,(u:length(ages)),]
  taxass=apply(taxass,c(1),mean) 
  for (i in (u:length(ages))){
    tise2c[,i,]=taxass
  }
  
  cal1c=tise1c
  cal2c=tise2c
  
  for (i in 1:length(unique(dimnames(tise1c)[[2]]))){
    cal1c[,i,]=tise1c[,i,]/ise1c
    cal2c[,i,]=tise2c[,i,]/ise2c
  }
  
  i=1
  for (i in 1:length(unique(dimnames(tise1c)[[1]]))){
    cal1c[i,,]=loess(cal1c[i,,]~names(cal1c[i,,]))$fitted
    cal2c[i,,]=loess(cal2c[i,,]~names(cal2c[i,,]))$fitted
  }
  
  cal1c[cal1c<0]=0
  cal2c[cal2c<0]=0
  
  calc=array(0,c(length(unique(dimnames(cal1c)[[1]])),2,length(unique(dimnames(cal1c)[[2]]))),dimnames=list(unique(dimnames(cal1c)[[1]]),c("1","6"),unique(dimnames(cal1c)[[2]])) )
  calc[,1,]=cal1c
  calc[,2,]=cal2c
  
  calcc=array(0,c(length(unique(dimnames(calc)[[1]])),length(unique(dimnames(calc)[[2]])),length(unique(dimnames(calc)[[3]])),nn),dimnames=list(unique(dimnames(calc)[[1]]),unique(dimnames(calc)[[2]]),unique(dimnames(calc)[[3]]),(year+1):(year+nn)) )
  for (i in 1:nn){
    calcc[,,,i]=calc}
  
  e=calcc
  for (i in 1:length(unique(dimnames(calcc)[[3]]))){
    e[,,i,]=calcc[,,i,]*isetpp 
  }
  return(e)
}




#' Funci\enc{ó}{o}n que proyecta las tasas de emigraci\enc{ó}{o}n interna (entre los espacios de proyecci\enc{ó}{o}n)
#'
#' Esta función proyecta las tasas de emigración interna en todo el período de proyección. En ella se considera que las tasas
#' se pueden descomponer para cada período t, en el producto de tres factores: la intensidad de la emigración interior por sexo desde el espacio h, en términos del Índice
#' sintético de emigración de dicho espacio h y sexo,  del calendario por edad, sexo y espacio h del fenómeno de la emigración y de un coeficiente de reparto según
#' el espacio destino k desde el espacio origen h, según sexo y edad.
#'
#' @param emig.int data frame que contiene el espacio de proyección, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y los emigrantes interiores (espacio*sexo*edad*tiempo*poblacion*evento).
#' @param emig.intc data frame con la suma de los emigrantes interiores del espacio origen h al espacio destino k en los últimos n períodos (espacio origen*sexo*edad*espacio destino)
#' @param nn número de períodos que se van a proyectar.
#' @keywords demography
#' @export
#' @importFrom plyr adply
#' @importFrom stats loess
#' @seealso \code{\link[demogal]{ExtIn}}, \code{\link[demogal]{ExtEm}}.
#' @return eic array con las tasas proyectadas de emigración interna por origen y destino (espacio origen*sexo*edad*espacio destino*tiempo)
#' @references Instituto Galego de Estatística
#' @encoding UTF-8
#' @examples
#' 
#' nn=15
#' u=IntEm(emig.int,emig.intc,nn)
#' 


IntEm=function(emig.int,emig.intc,nn){
  year=max(emig.int[,4])+1
  arr1=d2a(list(unique(emig.int[,1]),unique(emig.int[,2]),sort(unique(emig.int[,3])),unique(emig.int[,4])),emig.int)
  arr2=d2a(list(unique(emig.int[,1]),unique(emig.int[,2]),sort(unique(emig.int[,3])),unique(emig.int[,1])),emig.intc)
  emig.int=arr1[[2]]
  pop=arr1[[1]]
  emig.intcc=arr2

  emig_int.t=apply(emig.int,c(1,3,4),sum) 
  pop.t=apply(pop,c(1,3,4),sum) 
  emig_int.tt=apply(emig_int.t,c(1,2),sum) 
  #pop.tt=pop.t[,,kk]
  pop.tt=apply(pop.t,c(1,2),sum) 
  
  a=data.frame(adply(emig_int.t,c(1,2,3)))
  aa=data.frame(adply(pop.t,c(1,2,3)))
  ages=as.numeric(as.character(unique(a[,2])))

  L= merge(aa,a,by=names(aa)[c(1,2,3)],all.x=TRUE)
  L=numero(L,3)
  iseint=tr(L)[[1]]
  
  a=data.frame(adply(emig_int.tt,c(1,2)))
  aa=data.frame(adply(pop.tt,c(1,2)))

  L= merge(aa,a,by=names(aa)[c(1,2)],all.x=TRUE)
  L=numero(L,2)
  L=data.frame(L,year)
  L[is.na(L)]=0
  
  iseintt=tr(L[,c(1,2,5,3,4)])[[1]]
  
  emig_int1=apply(emig.int[,1,,],c(1,2),sum)
  pop1=apply(pop[,1,,],c(1,2),sum)
  emig_int2=apply(emig.int[,2,,],c(1,2),sum)
  pop2=apply(pop[,2,,],c(1,2),sum)
  a=data.frame(adply(emig_int1,c(1,2)))
  aa=data.frame(adply(pop1,c(1,2)))

  L= merge(aa,a,by=names(aa)[c(1,2)],all.x=TRUE)
  L=numero(L,2)
  L=data.frame(L,year)
  L[is.na(L)]=0
  iseint1=tr(L[,c(1,2,5,3,4)])[[1]]
  
  a=data.frame(adply(emig_int2,c(1,2)))
  aa=data.frame(adply(pop2,c(1,2)))
  
  L= merge(aa,a,by=names(aa)[c(1,2)],all.x=TRUE)
  L=numero(L,2)
  L=data.frame(L,year)
  L[is.na(L)]=0
  
  iseint2=tr(L[,c(1,2,5,3,4)])[[1]]
  
  de1=iseint1/iseintt
  de2=iseint2/iseintt
  
  #calculamos indices sintéticos de migración por sexo
  
  iset1=iseint[,ncol(iseint)]*de1
  iset2=iseint[,ncol(iseint)]*de2
  iset=cbind(iset1,iset2)
  
    #Calculamos calendarios
  
  taxas=apply(emig.int,c(1,2,3),sum)/apply(pop,c(1,2,3),sum)
  
  ###Tomamos as taxas a partir dos 85 anos como a media
  u=findInterval(85,ages)
  
  taxass=taxas[,,(u:length(ages))]
  taxass=apply(taxass,c(1,2),mean) 
  for (i in (u:length(ages))){
    taxas[,,i]=taxass
  }
  
  taxas[is.na(taxas)]=0
  taxas[taxas=='Inf']=0
  
  iseintc=array(0,c(length(unique(dimnames(iseint1)[[1]])),2),dimnames=list(unique(dimnames(iseint1)[[1]]),c("1","6")) )
  iseintc[,1]=iseint1
  iseintc[,2]=iseint2
  
  calc=taxas 
  for (i in 1:length(dimnames(taxas)[[3]])){
    calc[,,i]=taxas[,,i]/iseintc
  }
  
  
  for (i in 1:length(unique(dimnames(calc)[[1]])))  {
    for (j in 1:length(unique(dimnames(calc)[[2]]))){
      calc[i,j,]= loess(calc[i,j,]~names(calc[i,j,]))$fitted
      
     
    }}
  
  ##calculamos coeficientes de reparto
  
  popc=apply(pop,c(1,2,3),sum) 
  
  temig.intcc=emig.intcc
  
  for (i in 1:length(unique(dimnames(emig.intcc)[[4]]))){
    temig.intcc[,,,i]= emig.intcc[,,,i]/popc
  }
  
  emig.intcs=apply(emig.intcc,c(1,2,3),sum)
  
  temig.intcs= emig.intcs/popc
  
  coef=temig.intcc
  for (i in 1:length(unique(dimnames(coef)[[4]]))){
    coef[,,,i]= temig.intcc[,,,i]/temig.intcs
  }
  
  coef[is.nan(coef)]=0
  coef[is.na(coef)]=0
  ei=coef
  for (i in 1:length(unique(dimnames(coef)[[4]]))){
    ei[,,,i]= coef[,,,i]*calc
  }
  
  for (i in 1:length(unique(dimnames(coef)[[4]]))){
    for (j in 1:length(unique(dimnames(coef)[[3]]))){
      ei[,,j,i]= ei[,,j,i]*iset
    }}
  
  eic=array(0,c(length(unique(dimnames(ei)[[1]])),length(unique(dimnames(ei)[[2]])),length(unique(dimnames(ei)[[3]])),length(unique(dimnames(ei)[[4]])),nn),dimnames=list(unique(dimnames(ei)[[1]]),unique(dimnames(ei)[[2]]),unique(dimnames(ei)[[3]]),unique(dimnames(ei)[[4]]),(year+1):(year+nn)) )
  for (i in 1:nn){
    eic[,,,,i]=ei}
  
  return(eic)
  
}



#Funciónn que chequea que os arrays teñan todos os elementos

chek=function(name,data){
  
  tablabase.g<-expand.grid(name)
  names(tablabase.g)=names(data)[1:length(name)]
  res=merge(tablabase.g,data,by=names(tablabase.g),all.x=TRUE)
  res<-res[order(res[,3],res[,2],as.numeric(res[,1])),1:ncol(res)]
  res[,ncol(res)]=ifelse(is.na(res[,ncol(res)]),0,res[,ncol(res)])
  return(res) 
  
}

