#' Funci\enc{ó}{o}n que calcula la tabla de mortalidad
#'
#' Esta función permite calcular las funciones de la tabla de mortalidad completa o abreviada por sexo,
#' para diferentes espacios geográficos y diferentes períodos de tiempo a partir de los datos 
#' de defunciones y población.
#'
#' @param data data frame que contiene el espacio, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo, la población y las defunciones (espacio*sexo*edad*tiempo*poblacion*evento).
#' @param imn data frame que contiene el período de tiempo, el espacio geográfico y el índice de masculinidad al nacimiento. Este argumento es necesario en el caso de no introducir los factores de separación.
#' @param tmi data frame que contiene el período de tiempo, el espacio geográfico, el sexo y la tasa 
#' de mortalidad infantil en tanto por uno. Si es nulo, se toman los valores aproximados a partir de las defunciones y la población del argumento data.
#' @param sep data frame que contiene el período de tiempo, el espacio geográfico, el sexo, 
#' el extremo inferior del intervalo de edad y los factores de separación. Si es nulo se toman los valores aproximados a partir de los tabulados por Coale-Demeny para los primeros intervalos y la mitad de la longitud del intervalo para el resto.
#' @param gedad vector que contiene el extremo inferior de los intervalos de edad. Si no se especifican, se toman los extremos inferiores de los intervalos de edades especificados en el data
#' @param reg valor alfanumérico de longitud 1 que indica el tipo de espacio bajo el que se aplican los valores de Coale-Demeny. Los valores posibles son W=OESTE, E=ESTE,
#' N=NORTE y S=SUR. Si no se especifica, se toma el valor W que es el que corresponde a España.
#' @keywords demography
#' @export
#' @importFrom stats lm
#' @importFrom utils head
#' @seealso \code{\link[demogal]{PMortality}}, \code{\link[demogal]{PFertility}}.
#' @return resul array donde los elementos son las tablas de mortalidad correspondientes a los diferentes espacios, períodos y sexo
#' @references Instituto Galego de Estatística
#' @encoding UTF-8
#' @examples
#' 
#' tm=mortable(m.data)
#' 

mortable<-  
 function(data, imn=NULL, tmi=NULL, sep=NULL, gedad=sort(unique(data[,3])),  reg="W") {
   # Predeclaración CRAN check
   # a <- NULL
   # ·························
   
   reg=toupper(reg)
    stopifnot(length(reg)==1 & reg %in% c("W","N","E","S"))
    stopifnot(ncol(data)==6,
      all(data[,2] %in% c(1,6)) | (all(data[,2] %in% c(0,1,6)) & !(is.null(sep) & is.null(imn))),
      is.numeric(data[,3]),is.numeric(data[,5]),is.numeric(data[,6])
    )
    
    data[[3]]= sapply(data[[3]],function(v)max(gedad[gedad<=v]))
    espacio=sort(unique(data[[1]]))
    longespacio=length(espacio)
    sexo=sort(unique(data[[2]]))
    longsexo=length(sexo)
    tiempo=sort(unique(data[[4]]))
    longtiempo=length(tiempo)
    longx=length(gedad)
    #arrs= d2a(list(espacio, sexo, gedad, tiempo), data)
    #POB=arrs[[1]]
    #DEF=arrs[[2]]
    bylist=list(factor(data[[1]],levels=espacio),factor(data[[2]],levels= sexo),
                factor(data[[3]],levels= gedad),factor(data[[4]],levels= tiempo))
    POB=tapply(data[[5]], INDEX=bylist, FUN=sum)
    DEF=tapply(data[[6]], INDEX=bylist, FUN=sum)
    TMI=NULL
    
    #inicializamos el resultado
    v<-c("x","n","a","D","P","m","q","p","l","d","L","T","e")
    RES=array(NA, dim=c(longespacio,longsexo, longx,longtiempo, 13))
    dimnames(RES)=list( espacio, sexo, gedad, tiempo, v)
    # y cubrimos las primeras variables

    ## rellenamos x:
    RES[, , , ,"x" ]= xar(xar(xar(gedad,2,longtiempo),1,longsexo),1,longespacio)
     
    ## rellenamos n:
    RES[ , ,1:(longx-1), ,"n"]=xar(xar(xar(gedad[2:longx]-gedad[1:(longx-1)],2,longtiempo),1,longsexo),1,longespacio)
    
    
    if(! is.null(sep)) {
      
      stopifnot( ncol(sep)==5,
                 identical(espacio, sort(unique(sep[[1]]))),
                 identical(sexo, sort(unique(sep[[2]]))),
                 identical(tiempo, sort( unique(sep[[4]]))),
                 is.numeric(sep[[3]]),
                 is.numeric(sep[[5]])
      )
      if(!identical(gedad, sort(unique(sep[[3]])))){
        stopifnot(identical(gedad, sort(unique(sep[[3]]))[1:length(gedad)]))
        sep<-subset(sep, a <= max(gedad))
        sep[sep[[4]]==max(gedad),5]<-NA
        }

    
      RES[ , , , ,"a"]=d2a(list(espacio, sexo, gedad, tiempo), sep)
      
       
    }
    else { #cuando no hay sep 
      # calculamos los sep
      
      CDa=array(c(0.33,1.352,0.35,1.361,0.33,1.558,0.35,1.57,
                  0.29,1.313,0.31,1.324,0.33,1.24,0.35,1.239),dim=c(2,2,4),
                dimnames=list( c("0a1","1a5"),1:2,c("W","N","E","S")))
      CDalfa=array(c(0.0425,1.653,0.05,1.524,0.0425,1.859,0.05,1.733,
                     0.0025,1.541,0.01,1.402,0.0425,1.614,0.05,1.487),dim=c(2,2,4),
                   dimnames=list( c("0a1","1a5"),1:2,c("W","N","E","S")) )
      CDbeta=array(c(2.875,3.013,3,1.627),dim=c(2,2),dimnames=list( c("0a1","1a5"),1:2))

      # Calculo de CD2: para cada tiempo y espacio, obtengo la tabla CD aplicable con el sexo ambolos calculado mediante el IMN correspondiente
      # y quedandome ademas solo con la region aplicable al espacio en cuestion. Tendra dims:
      # c("a","alfa", "beta") c("0a1","1a5") , c("0","1","6") ,  espacio, tiempo
      CDa=CDa[ , ,reg]
      CDalfa=CDalfa[ , ,reg]
      CD2=array(dim=c(3,2,longespacio,3,longtiempo))
      dimnames(CD2)=list(c("a","alfa", "beta"), c("0a1","1a5") , espacio,c("0","1","6") , tiempo)
      CD2[1, , ,2:3, ]=xar(xar(CDa,2,longespacio),4,longtiempo)
      CD2[2, , ,2:3, ]=xar(xar(CDalfa,2,longespacio),4,longtiempo)
      CD2[3, , ,2:3, ]=xar(xar(CDbeta,2,longespacio),4,longtiempo)
      
      if( 0 %in% sexo){
        stopifnot(! is.null(imn))
        stopifnot(espacio == sort(unique(imn[[1]])),tiempo == sort( unique(imn[[2]])),is.numeric(imn[[3]]))          
        IMN=d2a(list(espacio,tiempo), imn)
        IMN2=xar(xar(IMN,1,2),1,3)
        CD2[ , , ,1,  ] = (IMN2*CD2[ , , ,2, ]+100*CD2[ , , ,3, ])/(IMN2+100)

      }
      #limitamos CD2 a los sexos realmente existentes
      CD2=CD2[,,,as.character(sexo), ,drop=FALSE]
      
      if(! is.null(tmi)) {
        
        stopifnot( ncol(tmi)==4,
                   espacio == sort(unique(tmi[[1]])),
                   sexo == sort(unique(tmi[[2]])),
                   tiempo == sort( unique(tmi[[3]])),
                   is.numeric(tmi[[4]])
        )
        
        TMI=d2a(list(espacio,sexo, tiempo),tmi)

      }
      else {
        M= DEF[ , ,1, ]/POB[ ,  ,1,  ]
        M[is.na(M)]=0
        M[M=="Inf"]=0
        TMI = M/(1+(1-  CD2[ 1,1, , , ] )*M)
      }
      
     
      
      # rellenamos "a"
      abreviada= all(head(gedad,3)==c(0,1,5))
      RES[ , ,1, ,"a"]= ifelse(TMI >= 0.1 , CD2[1,1, , , ], CD2[2,1, , , ]+ CD2[3,1, , , ]*TMI)
      if(abreviada)
        RES[ , ,2, ,"a"]= ifelse(TMI>= 0.1 , CD2[1,2, , , ], CD2[2,2, , , ]- CD2[3,2, , , ]*TMI)
      else
        RES[ , ,2, ,"a"]= RES[ , ,2, ,"n"]/2  
      RES[ , ,3:longx, ,"a"]= RES[ , ,3:longx, ,"n"]/2 
  
    } #fin else
   

    ## rellenamos "D" y "P":
    RES[ , , , ,"D"]= DEF
    RES[ , , , ,"P" ]= POB
    ## rellenamos "m":
    RES[ , , , ,"m"]=ifelse(POB==0, 0, DEF/POB) 
    ## rellenamos q:
    if(is.null(tmi))
      RES[  , ,1, ,"q" ]=  RES[ , ,1, ,"m"]/(1+(1-RES[ , ,1 , ,"a"])*RES[ , ,1, ,"m"]) 
    else {if(is.null(TMI)) TMI=d2a(list(espacio,sexo,tiempo),tmi)
      RES[  , ,1, ,"q"]=TMI}
    
    RES[ , ,2:(longx-1), ,"q"]= RES[ , ,2:(longx-1), ,"n"]* RES[ , ,2:(longx-1), ,"m"]/
      (1+( RES[ , ,2:(longx-1), ,"n"]- RES[ , ,2:(longx-1), ,"a"])* RES[ , ,2:(longx-1), ,"m"])
    RES[ , ,longx, ,"q"]= 1
    RES[ , ,2:longx, ,"q"]= pmin(RES[ , ,2:longx, ,"q"],1)
    
    ## rellenamos "p":
    RES[ , , , ,"p"]= 1- RES[ , , , ,"q"]
    ## rellenamos "l":
    RES[, ,1, ,"l"]=100000
    RES[ , ,2:longx, ,"l"]= aperm(apply(RES[ , ,1:(longx-1) , ,"p",drop=FALSE], c(1,2,4,5), cumprod),c(2,3,1,4,5))*100000
    ## rellenamos "d":
    RES[ , , , ,"d"]= RES[, , , ,"q"]* RES[ , , , ,"l"]
    ## rellenamos "L":
    RES[ , ,1:(longx-1) , ,"L"]= RES[ , ,1:(longx-1), ,"n"]* RES[ , ,2:longx , ,"l"]+ 
      RES[ , ,1:(longx-1) , ,"a"]* RES[ , ,1:(longx-1) , ,"d"]
    RES[ , ,longx, ,"L"]= ifelse(RES[ , ,longx, ,"m"]>0,     RES[ , ,longx, ,"d"]/ RES[ , ,longx, ,"m"], 0)
    ## rellenamos "T":
    RES[ , ,longx , ,"T" ]= RES[ , ,longx, ,"L"]
    RES[ , ,(longx-1):1 , ,"T"]= aperm(apply(RES[ , ,(longx-1):1 , ,"L",drop=FALSE], c(1,2,4,5), cumsum),c(2,3,1,4,5))+ RES[ , ,rep(longx,longx-1), ,"L",drop=FALSE]
    ## rellenamos "e"
    RES[ , , , ,"e"]=ifelse(RES[, , , ,"l"]==0,0,RES[ , , , ,"T"]/RES[ , , , ,"l"])
    names(RES)=list(espacio, sexo,gedad, tiempo,v)
    RES
  }
