#' Funci\enc{ó}{o}n que proyecta la poblaci\enc{ó}{o}n para el per\enc{í}{i}odo de proyecci\enc{ó}{o}n
#'
#' Esta función proyecta la población para el espacio de proyección, sexo, edad y todo el período de proyección. 
#'
#' @param pop data frame que contiene el espacio geográfico de proyección, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo y la población base de la proyección (espacio*sexo*edad*tiempo*poblacion).
#' @param relation data frame opcional que contiene las relaciones jerárquicas entre el espacio de la proyección y el espacio superior (primera columna, el espacio de la proyección; segunda columna, el espacio superior al de proyección). Las etiquetas de las filas serán las etiquetas del espacio de proyección.
#' @param r data frame que contiene el espacio de proyección, el sexo, los extremos inferiores de los intervalos de edades y la proporción de población promedio de los últimos n períodos de tiempo anteriores a la proyección que pasa de un grupo de edad a otro en el período siguiente (espacio*sexo*edad*proporcion).
#' @param nn número de períodos que se van a proyectar.
#' @param m.rate.pr array con las tasas de mortalidad proyectadas, salida de la función \code{\link[demogal]{PMortality}} o \code{\link[demogal]{PMortalityReg}}.
#' @param f.rate.pr array con las tasas de fecundidad proyectadas, salida de la función \code{\link[demogal]{PFertility}} o \code{\link[demogal]{PFertilityReg}}.
#' @param migration list con las proyecciones de las tasas de emigración exterior (salida de la función \code{\link[demogal]{ExtEm}} o \code{\link[demogal]{ExtEmReg}}), los inmigrantes procedentes del exterior (salida de la función \code{\link[demogal]{ExtIn}}) y las tasas de emigración interior (salida de la función \code{\link[demogal]{IntEm}}). Si no se dispone de las tasas de emigración interior este elemento de la lista se pone igual a NULL.
#' @param pop.sup.pr data frame opcional que contiene el espacio superior, el sexo, los extremos inferiores de los intervalos de edades, el período de tiempo y la población proyectada para el espacio superior (espacio superior*sexo*edad*tiempo*poblacion).
#' @param b.sup.pr data frame opcional que contiene el espacio superior, el sexo, el período de tiempo y los nacimientos proyectados para el espacio superior (espacio superior*sexo*tiempo*nacimientos).
#' @param m.sup.pr data frame opcional que contiene el espacio superior, el sexo, el período de tiempo y las defunciones proyectadas para el espacio superior (espacio superior*sexo*edad*tiempo*defunciones).
#' @keywords demography
#' @export
#' @seealso \code{\link[demogal]{PMortality}}, \code{\link[demogal]{PMortalityReg}}, \code{\link[demogal]{PFertility}}, \code{\link[demogal]{PFertilityReg}}.
#' @return poblacion array con la población proyectada (espacio*sexo*edad*tiempo)
#' @return births array con los nacimientos proyectados (espacio*sexo*tiempo)
#' @return deaths array con las defunciones proyectadas (espacio*sexo*edad*tiempo)
#' @references Instituto Galego de Estatística
#' @encoding UTF-8
#' @examples
#' 
#' nn=15
#' l.sup=PMortality(m.data.s,nn,reg="W")[[2]]
#' m.rate.pr=PMortalityReg(m.data,l.sup,relation,reg="W")
#' f.rate.pr=PFertilityReg(b.data,tfr.sup=NULL,nn,relation)
#' 
#' #EMIGRACIONES EXTERNAS
#' aux2=merge(emig.out,relation,by.x=names(emig.out)[1],by.y=names(relation)[1],all.x=TRUE)
#' emig.outt=aggregate(aux2[,c(5,6)],list(aux2[,7],aux2[,2],aux2[,3],aux2[,4]),sum)
#' names(emig.outt)=names(emig.out)
#' u=ExtEm(emig.outt,nn)
#' iset=u[[2]]
#' ise=u[[3]]
#' u=ExtEmReg(emig.out,iset,ise,relation,nn)
#' 
#' #INMIGRACIONES PROCEDENDES DEL EXTERIOR
#' v=ExtIn(imig.out,relation,nn,v_inmig=NULL)
#' 
#' #EMIGRACIONES INTERNAS
#' w=IntEm(emig.int,emig.intc,nn)
#' migration=list(u,v,w)
#' 
#' r<-expand.grid(unique(pop[,1]),unique(pop[,2]),unique(pop[,3])[2:21])
#' r[,4]=0.2
#' names(r)=c("comarca","sexo","Idadeq","dato")
#' m=projection(nn,r,pop,m.rate.pr,f.rate.pr,migration,relation=NULL)
#'
#' 
#'  

projection=function (nn,r,pop,m.rate.pr,f.rate.pr,migration,relation=NULL,pop.sup.pr=NULL,b.sup.pr=NULL,m.sup.pr=NULL){
 
  if (!(dim(f.rate.pr)[3]==nn)) stop ("El per\u00EDodo de las tasas de fecundidad proyectadas no coincide con el n\u00FAmero de per\u00EDodos que se quieren proyectar")
  if (!(dim(m.rate.pr)[4]==nn)) stop ("El per\u00EDodo de las tasas de mortalidad proyectadas no coincide con el n\u00FAmero de per\u00EDodos que se quieren proyectar")
  
  year=pop[,4][1]
  
  if (!(year==as.numeric(as.character(dimnames(m.rate.pr)[[4]][1])))) stop ("El per\u00EDodo del fichero de poblaci\u00F3n debe coincidir con el primer per\u00EDodo del fichero de mortalidad")
  if (!(year==as.numeric(as.character(dimnames(f.rate.pr)[[3]][1])))) stop ("El per\u00EDodo del fichero de poblaci\u00F3n debe coincidir con el primer per\u00EDodo del fichero de fecundidad")
  r=numero(r,3)
  r=d2a(list(unique(r[,1]),unique(r[,2]),unique(r[,3])),r)
 
  #Convertimmos o dataframe da poboacion base nun array:
  pop=numero(pop,4)
  pob_f=d2a(list(sort(unique(pop[,1])),sort(unique(pop[,2])),sort(unique(pop[,3]))),pop)[[2]]
  ano_ini=year
  u=names(pop)[1:3]
  poblacion=list()
  i=1
  n=array(0,dim=c(dim(pob_f)[1],dim(pob_f)[2],nn),dimnames=list(unlist(dimnames(pob_f)[1]),unlist(dimnames(pob_f)[2]),((year):(year+nn-1))))
  m=array(0,dim=c(dim(pob_f)[1],dim(pob_f)[2],dim(pob_f)[3],nn),dimnames=list(unlist(dimnames(pob_f)[1]),unlist(dimnames(pob_f)[2]),unlist(dimnames(pob_f)[3]),((year):(year+nn-1))))
  for (i in 1:(nn)){
       
    #MIGRACIONES
    t_emig_fora=migration[[1]][,,,i]
    dim(t_emig_fora)=dim(migration[[1]])[1:3]
    dimnames(t_emig_fora)=dimnames(migration[[1]])[1:3]
    INF=migration[[2]][,,,i]
    dim(INF)=dim(migration[[2]])[1:3]
    dimnames(INF)=dimnames(migration[[2]])[1:3]
    t_emig_int=migration[[3]]
    
    #MORTALIDAD 
  
    mulm=m.rate.pr[,,,i]*pob_f[,,]  #Aqui le puse los corchetes a la pob_f porque la dimensión es distinta si se le pone
                                   #corchetes y no ( porque elimina la primera dimensión que es 1)
    dim(mulm)=dim(pob_f)
    dimnames(mulm)=dimnames(pob_f)
    matriz_tm=mulm
    #AjUSTAMOS LOS DATOS DE MORTALIDAD
    if (!is.null(relation) & !is.null(m.sup.pr)){
    datos_defu_1=m.sup.pr[m.sup.pr[,4]==year,]
    m.sup=d2a(list(sort(unique(datos_defu_1[,1])),sort(unique(datos_defu_1[,2])),sort(unique(datos_defu_1[,3]))),datos_defu_1)[[2]]
    matriz_tm=calibrar(mulm,m.sup,relation=relation)
    mulm=matriz_tm
     }
    #######
   
    matriz_tm=matriz_tm/pob_f
    matriz_tm[is.na(matriz_tm)]=0
    matriz_tm[matriz_tm[,,]=='Inf']=0
    
    #NACIMIENTOS
    F_prox_1=f.rate.pr[,,i]
    dim(F_prox_1)<-c(dim(f.rate.pr)[1:2],1)
    dimnames(F_prox_1)<-c(dimnames(f.rate.pr)[1:2],year)
    ida=intersect(dimnames(m.rate.pr)[3][[1]],dimnames(F_prox_1)[2][[1]])
    pob_fe=pob_f[,2,ida]
    dim(pob_fe)=dim(F_prox_1)
    
    mul=pob_fe*F_prox_1
    
    nacc=cbind(rowSums(mul)*0.51563997,rowSums(mul)*0.484360003)
    colnames(nacc)=c("1","6")
    rownames(nacc)=rownames(mul)
       
    ##AJUSTE NACIMIENTOS
    if (!is.null(relation) & !is.null(b.sup.pr)){    
    b.sup.pr1=b.sup.pr[b.sup.pr[,3]==(year),]
    u=d2a(list(sort(unique( b.sup.pr1[,1])),sort(unique(b.sup.pr1[,2]))),b.sup.pr1[,c(1,2,4)])
    nacc=calibrar(nacc,u,relation=relation)
      }
    
    if(is.null(INF)==FALSE & is.null(t_emig_fora)==FALSE){
    EMi=array(0,dim=dim(INF),dimnames=dimnames(INF))
    INi=array(0,dim=dim(INF),dimnames=dimnames(INF))}
    else {
    EMi=array(0,dim=dim(pob_f),dimnames=dimnames(pob_f))
    INi=array(0,dim=dim(pob_f),dimnames=dimnames(pob_f))
    INF=array(0,dim=dim(pob_f),dimnames=dimnames(pob_f))
    t_emig_fora=array(0,dim=dim(pob_f),dimnames=dimnames(pob_f))}
    
    #Facemos o calculo de P1
    pr_f=P1(r,matriz_tm,t_emig_fora,INF,EMi,INi,pob_f,nacc)
  
    eps=0.01
    dif=1
    
    ###PROCEDEMENTO ITERATIVO SI LAS INMIGRACIONES INTERNAS SON NO NULAS
    if (!is.null(migration[[3]])){
    
    while (dif>eps){
      
    p4i=emi.imi.int(i,t_emig_int,pr_f,pob_f,nacc)[[1]]
    p4e=emi.imi.int(i,t_emig_int,pr_f,pob_f,nacc)[[2]]
    
    #Facemos o calculo de P5
    p5=P1(r,matriz_tm,t_emig_fora,INF,p4e,p4i,pob_f,nacc)
    a=abs(p5-pr_f)/pr_f
    dif=max(abs(p5-pr_f)/pr_f)
    pr_f=p5   
    }
    pob_f=p5
    }
    else
    {pob_f=pr_f}  
   
    #AJUSTAMOS POBLACION
    
    if (!is.null(relation) & !is.null(pop.sup.pr)){
    #Convertimos o array en dataframes
    pop.sup.pr1=pop.sup.pr[pop.sup.pr[,4]==(year+1),]
    u=d2a(list(sort(unique( pop.sup.pr1[,1])),sort(unique( pop.sup.pr1[,2])),sort(unique( pop.sup.pr1[,3]))), pop.sup.pr1[,c(1,2,3,5)])
    pob_f=calibrar(pob_f,u,relation=relation)
    
    }
    year=year+1
    poblacion[[i]]=pob_f
    m[,,,i]=mulm
    n[,,i]=nacc

  } 
  
  names(poblacion)=paste("Poblacion.proyectada",(ano_ini+1):year,sep=".")
  return(list("Poblacion.proyectada"=poblacion,"births"=n,"deaths"=m))
  #final do bucle
}

#' Funci\enc{ó}{o}n que eval\enc{ú}{u}a las expresiones del modelo de proyecci\enc{ó}{o}n multiregional para un instante del per\enc{í}{i}odo proyectivo 
#'
#' Esta función evalúa las expresiones del modelo de proyección multirregional para 
#' el cálculo de la población proyectada en un instante del período proyectivo
#' 
#' @param matriz.tm  array con las tasas de mortalidad proyectadas (espacio*sexo*edad) obtenidas de \code{\link[demogal]{PMortality}} o \code{\link[demogal]{PMortalityReg}} .
#' @param t.emig.out array con las tasas de emigración al exterior proyectadas (espacio*sexo*edad) obtenidas de \code{\link[demogal]{ExtEm}}.
#' @param r array con los datos de la proporción de población promedio de los últimos n períodos de tiempo anteriores
#'  a la proyección según espacio, sexo y grupo de edad que pasa de un grupo de edad a otro en el período siguiente (espacio*sexo*edad*tiempo).
#' @param INF array con los inmigrantes procedentes del exterior proyectados obtenidos de \code{\link[demogal]{ExtIn}} (espacio*sexo*edad).
#' @param EMi array con los emigrantes interiores proyectados a partir de las taxas de emigración interior obtenidas de \code{\link[demogal]{IntEm}} (espacio*sexo*edad).
#' @param INi array con los inmigrantes interiores proyectados proyectados a partir de las taxas de emigración interior obtenidas de \code{\link[demogal]{IntEm}} (espacio*sexo*edad).
#' @param pob_f array con la población del período anterior (espacio*sexo*edad). 
#' @param nac array con los nacimientos proyectados para el período anterior al que se proyecta (espacio*sexo).
#' @keywords demography
#' @export
#' @importFrom abind abind
#' @seealso \code{\link[demogal]{PFertility}}, \code{\link[demogal]{PMortality}}.
#' @return pr_f array con la población proyectada (espacio*sexo*edad*tiempo)
#' @references Instituto Galego de Estatística
#' @encoding UTF-8
#' 


P1=function(r,matriz.tm,t.emig.out,INF,EMi,INi,pob_f,nac){
  ##todos excepto 1-4, 0 e 100 e mas
  
  cc=dimnames(matriz.tm)[[3]][3:(length(dimnames(matriz.tm)[[3]])-1)]

  ccc=dimnames(matriz.tm)[[3]][2:(length(dimnames(matriz.tm)[[3]])-2)]
  
  a=matriz.tm[,,dimnames(matriz.tm)[[3]] %in% cc]
  b=t.emig.out[,,dimnames(t.emig.out)[[3]] %in% cc]
  d=INF[,,dimnames(INF)[[3]] %in% cc]
  e=EMi[,,dimnames(EMi)[[3]] %in% cc]
  g=INi[,,dimnames(INi)[[3]] %in% cc]
  f=pob_f[,,dimnames(pob_f)[[3]] %in% cc]
  aa=matriz.tm[,,dimnames(matriz.tm)[[3]] %in% ccc]
  bb=t.emig.out[,,dimnames(t.emig.out)[[3]] %in% ccc]
  dd=INF[,,dimnames(INF)[[3]] %in% ccc]
  ee=EMi[,,dimnames(EMi)[[3]] %in% ccc]
  gg=INi[,,dimnames(INi)[[3]] %in% ccc]
  ff=pob_f[,,dimnames(pob_f)[[3]] %in% ccc]
  mul=(((1-0.5*(aa+bb))*ff+dd+gg-ee)/(1+0.5*(a+b)))*(r[,,-1])+(((1-0.5*(a+b))*f+d+g-e)/(1+0.5*(a+b)))*(1-r[,,-1])
  dim(mul)=c(dim(pob_f)[1:2],dim(mul)[length(dim(mul))])
  
  #1-4
  
  num2=((1-0.5*(matriz.tm[,,1]+t.emig.out[,,1]))*pob_f[,,1]+INF[,,1]+INi[,,1]-EMi[,,1])/(1+0.5*(matriz.tm[,,1]+t.emig.out[,,1]))
  num22=(((1-0.5*(matriz.tm[,,2]+t.emig.out[,,2]))*pob_f[,,2]+INF[,,2]+INi[,,2]-EMi[,,2])/(1+0.5*(matriz.tm[,,2]+t.emig.out[,,2])))*(1-r[,,1])  
  mul2=num2+num22 
  dim(mul2)<-c(dim(pob_f)[1:2],1)
  
  #0 anos
  mul3=((1-0.5*(matriz.tm[,,1]+t.emig.out[,,1]))*nac+INF[,,1]+INi[,,1]-EMi[,,1])/(1+0.5*(matriz.tm[,,1]+t.emig.out[,,1]))
  dim(mul3)<-c(dim(pob_f)[1:2],1)
  l=length(dimnames(matriz.tm)[[3]])
  
  #100 o mais
  num4=(((1-0.5*(matriz.tm[,,l-1]+t.emig.out[,,l-1]))*pob_f[,,l-1]+INF[,,l-1]+INi[,,l-1]-EMi[,,l-1])/(1+0.5*(matriz.tm[,,l-1]+t.emig.out[,,l-1])))*r[,,l-2]
  num44=((1-0.5*matriz.tm[,,l])*pob_f[,,l]+INF[,,l]+INi[,,l]-EMi[,,l])/(1+0.5*(matriz.tm[,,l]+t.emig.out[,,l])) 
  mul4=num4+num44  
  dim(mul4)<-c(dim(pob_f)[1:2],1)
  
  pr_f=abind(mul3,mul2,mul,mul4)
  
  dimnames(pr_f)=dimnames(pob_f)
  
  return(pr_f)
  
}

###FUNCIoN QUE CALCULA LAS TAXAS DE EMIGRACión INTERIORES

emi.imi.int=function(i,w,pr_f,pob_f,nacc){
  w=w[,,,,i]
  a=(pr_f+pob_f)/2
  
  ###Para os que teñen 0 anos
  a[,,1]=(nacc+pob_f[,,1])/2
  ww=w
  for (j in 1:(dim(w)[1])){
    ww[,,,j]=w[,,,j]*a
  }
  
  www=array(0,dim=dim(ww),dimnames=dimnames(ww))
  for (j in 1:(dim(ww)[1])){
    for (k in 1:(dim(ww)[4])){
      if (j==k){www[j,,,k]=ww[j,,,k]}
    }
   }
  ww=ww-www
  IN=apply(ww,c(2,3,4),sum) 
  INp=apply(IN,c(3,1,2),sum)
  EM=apply(ww,c(1,2,3),sum)
  

return(list(INp,EM))

}

 

numero=function(m,n){
  for (i in 1:n){
    m[,i]=as.numeric(as.character(m[,i]))
  }
  return(m)
}